--[[==================================================================
Auto Track Colors

Changes the colors of grouped tracks according to their group color 
and applies different shades.
Track colors change whenever their group color changes or
when a track is inserted/moved into a group.
Vice versa, when a new group is inserted, its color will be changed
according to the member track it was inserted to.

Author: Martin Dombrowa [md@strobotone.de]
Version: 1.0
==================================================================]]--


-- prefs ----------------------------------------------------------------------


local prefs = renoise.Document.create("ScriptingToolPreferences") 
{
  dim1 = 0.85,  -- relative color-brigthness of tracks in group
  dim2 = 0.93   -- color-brightness falloff across tracks in group
}

renoise.tool().preferences = prefs


--menu entries / keybindings -----------------------------------------


renoise.tool():add_menu_entry 
{
  name = "Pattern Editor:Set All Track Colors From Group",
  invoke = function() 
      set_all_track_colors_from_group(prefs.dim1, prefs.dim2)
    end
}

renoise.tool():add_menu_entry 
{
  name = "Mixer:Set All Track Colors From Group",
  invoke = function()  
      set_all_track_colors_from_group(prefs.dim1, prefs.dim2)
    end
}

renoise.tool():add_keybinding 
{
  name = "Pattern Editor:Tools:Set All Track Colors From Group",
  invoke = function() 
      set_all_track_colors_from_group(prefs.dim1, prefs.dim2)
    end
}


renoise.tool():add_menu_entry 
{
  name = "Main Menu:Tools:Auto Track Colors Config...",
  invoke = function()  
      open_config_dialog()
    end
}


-- functions ---------------------------------------------------------


function dim_color(rbg, v)
  return 
  { 
    rbg[1] * v, 
    rbg[2] * v, 
    rbg[3] * v 
  }
end 


function get_all_tracks()
  return renoise.song().tracks
end


function is_group(track)
  return track.type == renoise.Track.TRACK_TYPE_GROUP
end


function is_normal_track(track)
  return track.type == renoise.Track.TRACK_TYPE_SEQUENCER
end


function set_all_track_colors_from_group()
  local trks = get_all_tracks()

  for i = 1, #trks
  do
    local grp = trks[i]
    if (is_group(grp))
    then
    
      local trk_color = dim_color(grp.color, prefs.dim1)
      for i = #grp.members, 1, -1
      do
        local trk = grp.members[i]
        
        -- prevents notifier loops in case this is a nested group
        if (is_group(trk))
        then
         if (trk.color_observable:has_notifier(set_all_track_colors_from_group))
          then 
            trk.color_observable:remove_notifier(set_all_track_colors_from_group)
          end
        end
        
        trk.color = trk_color
        trk_color = dim_color(trk_color, prefs.dim2)
        
      end
      
    end
  end
  
end


function set_track_colors_from_group(group)
  
  local trk_color = dim_color(group.color, prefs.dim1)
  for i = #group.members, 1, -1
  do
    local trk = group.members[i]
    trk.color = trk_color
    trk_color = dim_color(trk_color, prefs.dim2)
  end

end


function set_track_color_from_group(group, track)
  
  local trk_color = dim_color(group.color, prefs.dim1)
  track.color = trk_color

end


function add_notifiers()
  --print("adding notifiers..")
  renoise.song().tracks_observable:add_notifier(on_tracks_changed)
  
  local trks = get_all_tracks()
  for i = #trks, 1, -1
    do
      local trk = trks[i]
      if (is_group(trk))
      then
        local grp = trk
        grp.color_observable:add_notifier(set_all_track_colors_from_group)
      end
  end

end


function remove_notifiers()
  --print("removing notifiers..")
  renoise.song().tracks_observable:remove_notifier(on_tracks_changed)
  
  local trks = get_all_tracks()
  for i = #trks, 1, -1
    do
      local trk = trks[i]
      if (is_group(trk))
      then
        local grp = trk
        grp.color_observable:remove_notifier(set_all_track_colors_from_group)
      end
  end

end


function on_tracks_changed(args)
  local trks = get_all_tracks()
  
  --rprint(args)
 

  local grp = nil
  local trk = nil
  
  if (args.type ~= "swap")
  then
  
    trk = trks[args.index]
    if (trk == nil) then return end

    if (is_group(trk))
    then
      
      grp = trk
 
      if (args.type == "remove")
      then
        --print("a group track was removed.")
        grp.color_observable:remove_notifier(set_all_track_colors_from_group)
      end
      
      if (args.type == "insert")
      then
        --print("a group track was added.")
        if (grp.color_observable:has_notifier(set_all_track_colors_from_group) == false)
        then 
          grp.color_observable:add_notifier(set_all_track_colors_from_group)
        end
        
        local col = grp.members[1].color
        grp.color = col
      end
      
    elseif (is_normal_track(trk))
    then
    
      if (args.type == "insert")
      then
        --print("a normal track was added.")
        grp = trk.group_parent
        if (grp ~= nil)
        then
          --print("a track was added to a group.")
          set_track_colors_from_group(grp)
        end
      elseif (args.type == "remove")
      then
        --print("a normal track was removed.")
      end
     
    end
    
  elseif (args.type == "swap")
  then
  
    print("a track was swapped.")
    trk = renoise.song().selected_track
    
    if (trk == nil) then return end
    
    grp = trk.group_parent
    if (grp ~= nil)
    then
      --print("a track was swapped in a group.")
      set_track_colors_from_group(grp, trk)
    end
    
  end

end


function update()
  set_all_track_colors_from_group()
end


-- notifiers ------------------------------------------------------------------


renoise.tool().app_release_document_observable:add_notifier(remove_notifiers)
renoise.tool().app_new_document_observable:add_notifier(add_notifiers)


-- GUI ------------------------------------------------------------------------

local dialog = nil

function open_config_dialog()

  if (dialog and dialog.visible) 
  then
    dialog:show() 
    return
  end

  local function keyhandler(dialog, key)
    if (key.modifiers == "" and key.name == "esc") 
    then dialog:close()
    end
  end
  
  local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
  local CONTENT_SPACING = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING  
  local CONTROL_WIDTH = 150
  local CONTROL_HEIGHT = 22
  local TEXT_WIDTH = 70
  local VALUE_WIDTH = 35
  
  local vb = renoise.ViewBuilder()
  
  -- show dlg
  
  local min1 = 0.25
  local min2 = 0.5
  local max = 1.0
  local tooltip1 = "Overall brigthness of tracks in group"  
  local tooltip2 = "Brightness falloff across tracks in group"
  
  dialog = renoise.app():show_custom_dialog("Auto Track Colors",
  
    vb:column
    {
       margin = DIALOG_MARGIN,
       spacing = CONTENT_SPACING,
    
       vb:row
       {
          vb:text 
          { 
            text = "Brightness",
            width = TEXT_WIDTH,
          },
          vb:slider
          {
            min = min1,
            max = max,
            bind = prefs.dim1,
            value = tonumber(prefs.dim1),
            notifier = function()
              update()
              end,
            width = CONTROL_WIDTH,
            tooltip = tooltip1
          },
          vb:valuefield
          {
            min = min1,
            max = max,
            bind = prefs.dim1,
            value = tonumber(prefs.dim1),
            notifier = function()
              update()
              end,
            width = VALUE_WIDTH,
          }
       },
       
       vb:row
       {
          vb:text 
          { 
            text = "Gradient",
            width = TEXT_WIDTH,
          },
          vb:slider
          {
            min = min2,
            max = max,
            bind = prefs.dim2,
            value = tonumber(prefs.dim2),
            notifier = function()
              update()
              end,
            width = CONTROL_WIDTH,
            tooltip = tooltip2
          },
          vb:valuefield 
          {
            min = min2,
            max = max,
            bind = prefs.dim2,
            value = tonumber(prefs.dim2),
              notifier = function()
              update()
              end,
            width = VALUE_WIDTH,
          }
       },
       
    },
   
    keyhandler 
  )


end




